#!/usr/bin/env python

__title__ = 'proxipy'
__description__ = 'Need proxy? Catch it.'
__url__ = 'http://github.com/sttlr/proxipy'
__version__ = '1.0'
__author__ = 'sttlr'
__license__ = 'MIT'
__copyright__ = 'Copyright 2018, sttlr'
